@@ -23,11 +23,10 @@
 
 #include <config.h>
 
-#include "gnome-i18nP.h"
+#include <glib/gi18n-lib.h>
 
 #include "gnome-exec.h"
 #include "gnome-util.h"
-#include "gnome-i18n.h"
 #include "gnome-gconfP.h"
 #include "gnome-init.h"
 
