@@ -1,4 +1,3 @@
-
 /*
  * Copyright (C) 1999, 2000 Red Hat, Inc.
  *               2001 SuSE Linux AG.
@@ -47,12 +46,11 @@
 #include <gconf/gconf-value.h>
 #include <gconf/gconf-client.h>
 
-#include "gnome-i18nP.h"
+#include <glib/gi18n-lib.h>
 
 #include "gnome-program.h"
 #include "gnome-util.h"
 #include "gnome-init.h"
-#include "gnome-i18n.h"
 #include "gnome-url.h"
 
 #include "libgnome-private.h"
@@ -903,7 +901,6 @@ gnome_program_locate_file (GnomeProgram *program, GnomeFileDomain domain,
     gchar *prefix_rel = NULL, *attr_name = NULL, *attr_rel = NULL;
     gchar fnbuf [PATH_MAX], *retval = NULL, **ptr;
     gboolean search_path = TRUE;
-    GValue value = { 0, };
 
     if (program == NULL)
 	program = gnome_program_get ();
@@ -1004,10 +1001,7 @@ gnome_program_locate_file (GnomeProgram *program, GnomeFileDomain domain,
     if (attr_name != NULL) {
 	gchar *dir;
 
-	g_value_init (&value, G_TYPE_STRING);
-	g_object_get_property (G_OBJECT (program), attr_name, &value);
-	dir = g_value_dup_string (&value);
-	g_value_unset (&value);
+	g_object_get (G_OBJECT (program), attr_name, &dir, NULL);
 
 	/* use the prefix */
 	if (dir == NULL) {
@@ -1296,6 +1290,7 @@ gnome_program_preinit (GnomeProgram *program,
     GnomeModuleInfo *a_module;
     poptContext argctx = NULL;
     int i;
+    char *prgname;
 
     g_return_val_if_fail (program != NULL, NULL);
     g_return_val_if_fail (GNOME_IS_PROGRAM (program), NULL);
@@ -1305,12 +1300,13 @@ gnome_program_preinit (GnomeProgram *program,
 	return NULL;
 
     /* Store invocation name */
-    g_set_prgname (argv[0]);
+    prgname = g_path_get_basename (argv[0]);
+    g_set_prgname (prgname);
+    g_free (prgname);
 
     /* 0. Misc setup */
     g_free (program->_priv->app_id);
     program->_priv->app_id = g_strdup (app_id);
-    g_set_prgname (app_id);
     g_free (program->_priv->app_version);
     program->_priv->app_version = g_strdup (app_version);
     program->_priv->argc = argc;
@@ -1427,6 +1423,130 @@ gnome_program_preinit (GnomeProgram *program,
     return argctx;
 }
 
+/**
+ * split_file_list:
+ * @str: a %G_SEARCHPATH_SEPARATOR separated list of filenames
+ *
+ * Splits a %G_SEARCHPATH_SEPARATOR-separated list of files, stripping
+ * white space and substituting ~/ with $HOME/.
+ *
+ * Stolen from pango.
+ *
+ * Return value: a list of strings to be freed with g_strfreev()
+ **/
+static char **
+split_file_list (const char *str)
+{
+  int i = 0;
+  int j;
+  char **files;
+
+  files = g_strsplit (str, G_SEARCHPATH_SEPARATOR_S, -1);
+
+  while (files[i])
+    {
+      char *file;
+      
+      file = g_strdup (files[i]);
+      g_strstrip (file);
+
+      /* If the resulting file is empty, skip it */
+      if (file[0] == '\0')
+	{
+	  g_free(file);
+	  g_free (files[i]);
+
+	  for (j = i + 1; files[j]; j++)
+	    files[j - 1] = files[j];
+
+	  files[j - 1] = NULL;
+
+	  continue;
+	}
+#ifndef G_OS_WIN32
+      /* '~' is a quite normal and common character in file names on
+       * Windows, especially in the 8.3 versions of long file names, which
+       * still occur now and then. Also, few Windows user are aware of the
+       * Unix shell convention that '~' stands for the home directory,
+       * even if they happen to have a home directory.
+       */
+      if (file[0] == '~' && file[1] == G_DIR_SEPARATOR)
+	{
+	  char *tmp = g_strconcat (g_get_home_dir(), file + 1, NULL);
+	  g_free (file);
+	  file = tmp;
+	}
+      else if (file[0] == '~' && file[1] == '\0')
+	{
+	  g_free (file);
+	  file = g_strdup (g_get_home_dir());
+	}
+#endif
+      g_free (files[i]);
+      files[i] = file;
+
+      i++;
+    }
+
+  return files;
+}
+
+static gchar **
+get_module_path (void)
+{
+  const gchar *module_path_env;
+  gchar *module_path;
+  static gchar **result = NULL;
+
+  if (result)
+    return result;
+
+  module_path_env = g_getenv ("GNOME_MODULE_PATH");
+
+  if (module_path_env)
+    module_path = g_build_path (G_SEARCHPATH_SEPARATOR_S,
+				module_path_env, LIBGNOME_LIBDIR, NULL);
+  else
+    module_path = g_build_path (G_SEARCHPATH_SEPARATOR_S,
+				LIBGNOME_LIBDIR, NULL);
+
+  result = split_file_list (module_path);
+  g_free (module_path);
+
+  return result;
+}
+
+static gchar *
+find_module (const gchar *name)
+{
+  gchar **paths;
+  gchar **path;
+  gchar *module_name;
+
+  if (g_path_is_absolute (name))
+    {
+      return g_strdup (name);
+    }
+
+  module_name = NULL;
+  paths = get_module_path ();
+  for (path = paths; *path; path++)
+    {
+      gchar *tmp_name;
+
+      tmp_name = g_module_build_path (*path, name);
+      if (g_file_test (tmp_name, G_FILE_TEST_EXISTS))
+        {
+          module_name = tmp_name;
+          goto found;
+        }
+      g_free (tmp_name);
+    }
+found:
+  g_strfreev (paths);
+  return module_name;
+}
+
 /**
  * gnome_program_module_load:
  * @mod_name: module name
@@ -1442,30 +1562,33 @@ gnome_program_module_load (const char *mod_name)
 {
     GModule *mh;
     const GnomeModuleInfo *gmi;
-    char tbuf[1024];
+    char *full_mod_name;
 
     g_return_val_if_fail (mod_name != NULL, NULL);
 
-    g_snprintf (tbuf, sizeof(tbuf), "lib%s.so.0", mod_name);
-
-    mh = g_module_open (mod_name, G_MODULE_BIND_LAZY);
-    if(!mh) {
-	g_snprintf (tbuf, sizeof(tbuf), "lib%s.so", mod_name);
-
-	mh = g_module_open (mod_name, G_MODULE_BIND_LAZY);
-    }
+    full_mod_name = find_module (mod_name);
+    if (!full_mod_name)
+      {
+        return NULL;
+      }
 
+    mh = g_module_open (full_mod_name, G_MODULE_BIND_LAZY);
     if (!mh)
-	return NULL;
+      {
+        return NULL;
+      }
 
-    if (g_module_symbol (mh, "dynamic_module_info", (gpointer *)&gmi)) {
+    if (g_module_symbol (mh, "dynamic_module_info", (gpointer *)&gmi)) 
+      {
 	gnome_program_module_register (gmi);
 	g_module_make_resident (mh);
 	return gmi;
-    } else {
+      } 
+    else 
+      {
 	g_module_close (mh);
 	return NULL;
-    }
+      }
 }
 
 /**
@@ -1492,14 +1615,16 @@ gnome_program_parse_args (GnomeProgram *program)
 	if (priv->goption_context) {
 		GError *error = NULL;
 		char **arguments;
+		int n_arguments;
 
 		/* We need to free priv->argv in finalize, but g_option_context_parse
 		 * modifies the array. So we just pass it like this.
 		 */
 		arguments = g_memdup (priv->argv, priv->argc * sizeof (char *));
+		n_arguments = priv->argc;
 
 		if (!g_option_context_parse (priv->goption_context,
-		     			     &priv->argc, &arguments,
+		     			     &n_arguments, &arguments,
 					     &error)) {
 			/* Translators: the first %s is the error message, 2nd %s the program name */
 			g_print(_("%s\nRun '%s --help' to see a full list of available command line options.\n"),
@@ -1725,6 +1850,11 @@ gnome_program_postinit (GnomeProgram *program)
  * pairs.
  *
  * Initialises the current GNOME libraries for use by the application.
+ * @app_id is used for the following purposes:
+ * - to find the programme's help files by gnome_help_*()
+ * - to load the app-specific gtkrc file from ~/.gnome2/$(APPID)rc
+ * - to load/save the app's accelerators map from ~/.gnome2/accelerators/$(APPID)
+ * - to load/save a GnomeEntry's history from gconf/apps/gnome-settings/$(APPID)/history-$(ENTRYID)
  *
  * Returns: A new #GnomeProgram instance representing the current application.
  * Unref the returned reference right before exiting your application.
