@@ -39,8 +39,8 @@
 
 #ifdef HAVE_ESD
 static char *esound_hostname = NULL;
-static gboolean esound_hostname_null_ok = FALSE;
 static int gnome_sound_connection = -1;
+static gboolean gnome_sound_enabled = FALSE;
 #endif
 
 typedef struct _sample
@@ -73,6 +73,8 @@ WAVFormatChunk;
 ( ( x & 0xff000000 ) >> 24 ) )
 #endif
 
+G_GNUC_INTERNAL void _gnome_sound_set_enabled (gboolean);
+
 /**
  * gnome_sound_sample_load_wav:
  * @file: filename to try loading a WAV file from.
@@ -284,8 +286,7 @@ gnome_sound_sample_load_wav(const char *file)
     }
   fclose (f);
   g_free (s);
-  if (s->data)
-    g_free (s->data);
+  g_free (s->data);
 
   return NULL;
 }
@@ -299,21 +300,37 @@ gnome_sound_sample_load_wav(const char *file)
 static gboolean
 use_sound (void)
 {
+  if (!gnome_sound_enabled)
+    return FALSE;
+
   if (gnome_sound_connection == -1){
-    if (esound_hostname || esound_hostname_null_ok){
       gnome_sound_connection = esd_open_sound (esound_hostname);
-      if (gnome_sound_connection == -1){
-	g_free (esound_hostname);
-	esound_hostname = NULL;
-	esound_hostname_null_ok = FALSE;
-	return FALSE;
-      }
-    }
   }
-  return TRUE;
+
+  return gnome_sound_connection >= 0;
+}
+
+static void
+close_sound_connection (void)
+{
+  if(gnome_sound_connection >= 0){
+    esd_close(gnome_sound_connection);
+    gnome_sound_connection = -1;
+  }
 }
 #endif
 
+void G_GNUC_INTERNAL
+_gnome_sound_set_enabled (gboolean enabled)
+{
+#ifdef HAVE_ESD
+  gnome_sound_enabled = enabled;
+  if (!enabled) {
+    close_sound_connection();
+  }
+#endif
+}
+
 #if defined(HAVE_LIBAUDIOFILE) && defined(HAVE_ESD)
 #include <audiofile.h>
 
@@ -522,10 +539,7 @@ gnome_sound_init(const char *hostname)
 #ifdef HAVE_ESD
 	srand(time(NULL));
 	g_free (esound_hostname);
-	if (hostname)
-		esound_hostname = g_strdup (hostname);
-	else
-		esound_hostname_null_ok = TRUE;
+	esound_hostname = g_strdup (hostname);
 #endif
 }
 
@@ -540,10 +554,8 @@ gnome_sound_shutdown(void)
 #ifdef HAVE_ESD
 	g_free (esound_hostname);
 	esound_hostname = NULL;
-	if(gnome_sound_connection >= 0){
-		esd_close(gnome_sound_connection);
-		gnome_sound_connection = -1;
-	}
+
+        close_sound_connection ();
 #endif
 }
 
